/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

public class STAXCallActionFactory implements STAXActionFactory
{
    private static String fDTDInfo =
"\n" +
"<!--================= The Call Element ============================= -->\n" +
"<!--\n" +
"     Perform a function with the referenced name.\n" +
"     The function attribute value is evaluated via Python.\n" +
"     Arguments can be specified as data to the call element.\n" +
"     Arguments are evaluated via Python.\n" +
"-->\n" +
"<!ELEMENT call       (#PCDATA)>\n" +
"<!ATTLIST call\n" +
"          function   CDATA    #REQUIRED\n" +
">\n" +
"\n";

    public String getDTDInfo()
    {
        return fDTDInfo;
    }

    public String getDTDTaskName()
    {
        return "call";
    }

    public STAXAction parseAction(STAX staxService, STAXJob job,
                                  org.w3c.dom.Node root) throws STAXException
    {
        String functionName = new String();
        STAXCallAction action = new STAXCallAction();
        
        NamedNodeMap rootAttrs = root.getAttributes();

        for (int i = 0; i < rootAttrs.getLength(); ++i)
        {
            Node thisAttr = rootAttrs.item(i);

            String errorInfo = "\n  Element: " + root.getNodeName() + 
                "  Attribute: " + thisAttr.getNodeName();

            if (thisAttr.getNodeName().equals("function"))
            {
                action.setFunction(STAXUtil.parseAndCompileForPython(
                    thisAttr.getNodeValue(), errorInfo));
            }
            else
            {
                throw new STAXInvalidXMLAttributeException(
                    root.getNodeName() + ": " + thisAttr.getNodeName());
            }
        }

        String errorInfo = "\n  Element: " + root.getNodeName();

        // Determine which call element is being processed
        
        action.setCallType(STAXCallAction.CALL_ONE_ARG);

        // Iterate its children nodes to get arguments passed in (if any)

        NodeList children = root.getChildNodes();
        
        for (int i = 0; i < children.getLength(); ++i)
        {
            Node thisChild = children.item(i);

            if (thisChild.getNodeType() == Node.COMMENT_NODE)
            {
                /* Do nothing */
            }
            else if (thisChild.getNodeType() == Node.TEXT_NODE)
            {
                action.setArgs(STAXUtil.parseAndCompileForPython(
                    thisChild.getNodeValue(), errorInfo));
            }
            else
            {
                throw new STAXInvalidXMLNodeTypeException(
                    Integer.toString(thisChild.getNodeType()));
            }
        }

        return action;
    }
}
