/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import java.util.TreeMap;

public class STAXHoldAction implements STAXAction
{
    private STAXHoldAction()
    { /* Do Nothing */ }

    public STAXHoldAction(String blockName)
    {
        fUnevalBlockName = blockName;
        fBlockName = blockName;
    }

    public String getXMLInfo()
    {
        return "<hold block=\"" + fBlockName + "\"/>";
    }

    public String getInfo()
    {
        return fBlockName;
    }

    public String getDetails()
    {
        return "BlockName:" + fBlockName;

    }

    public void execute(STAXThread thread)
    {
        thread.popAction();

        try
        {
            if (fUnevalBlockName == null)
                fBlockName = thread.pyStringEval("STAXCurrentBlock");
            else
                fBlockName = thread.pyStringEval(fUnevalBlockName);
        }
        catch (STAXPythonEvaluationException e)
        {
            thread.pySetVar("STAXPythonEvalMsg",
                            "\n" + getXMLInfo() + "\n\n" + e);
            thread.raiseSignal("STAXPythonEvaluationError");
            return;
        }

        TreeMap blockMap = (TreeMap)thread.getJob().getData("blockMap");

        if (!blockMap.containsKey(fBlockName))
        {
            thread.pySetVar("STAXBlockDoesNotExistMsg", "  " + 
                            getXMLInfo());
            thread.raiseSignal("STAXBlockDoesNotExist");
        }
        else
        {
            STAXBlockAction theBlock = (STAXBlockAction)blockMap.get(fBlockName);

            if (theBlock.getBlockState() == STAXBlockAction.BLOCK_RUNNING)
                theBlock.holdBlock();
        }
    }

    public void handleCondition(STAXThread thread, STAXCondition cond)
    {
        thread.popAction();
    }

    public STAXAction cloneAction()
    {
        STAXHoldAction clone = new STAXHoldAction();

        clone.fUnevalBlockName = fUnevalBlockName;
        clone.fBlockName = fBlockName;

        return clone;
    }

    private String fUnevalBlockName;
    private String fBlockName;
}
