/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;

public class STAXScriptAction implements STAXAction
{
    public STAXScriptAction()
    { /* Do Nothing */ }

    public STAXScriptAction(String value)
    {
        fValue = value;
    }

    public String getValue() { return fValue; } 
    public void setValue(String value) { fValue = value; }

    public String getXMLInfo()
    {
        return "<script>" + fValue + "</script>";
    }

    public String getInfo()
    {
        int valueLength = fValue.length();
        if (valueLength > 40)
            return fValue.substring(0, 40) + "...";
        else
            return fValue;
    }

    public String getDetails()
    {
        return "Value:" + fValue;
    }

    public void execute(STAXThread thread)
    { 
        try
        { 
            thread.pyExec(fValue);
            thread.popAction();
        }
        catch (STAXPythonEvaluationException e)
        {
            thread.popAction();

            thread.pySetVar("STAXPythonEvalMsg", 
                            "\n" + getXMLInfo() + "\n\n" + e);
            thread.raiseSignal("STAXPythonEvaluationError");
        }  
    }

    public void handleCondition(STAXThread thread, STAXCondition cond)
    {
        thread.popAction();
    }

    public STAXAction cloneAction()
    {
        STAXScriptAction clone = new STAXScriptAction();

        clone.fValue = fValue;

        return clone;
    }

    private String fValue = new String();
}
