/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2005                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAF.h"
#include "STAFProc.h"
#include "STAFFSCopyManager.h"
#include "STAFTimestamp.h"
#include "STAFFSService.h"
#include "STAFTrace.h"
#include "STAFServiceManager.h"


STAFRC_t STAFFSCopyManager::add(
    const STAFString &name, const STAFString &machine,
    unsigned int direction, unsigned int type, unsigned int mode,
    unsigned int fileSize, unsigned int &mapID)
{
    if (direction == kSTAFFSCopyTo)
    {
        if (STAFTrace::doTrace(kSTAFTraceServiceRequest) &&
            STAFServiceManager::doTraceService("FS"))
        {
            // Record a FS Copy Request Started trace message

            STAFString message = "FS Copy Request Started - From Machine: " +
                machine;
        
            if (type == kSTAFFSFileCopy)
            {
                message += ", To File: " + name;

                if (mode == kSTAFFSBinary)
                    message += ", Mode: Binary";
                else
                    message += ", Mode: Text";
            }
            else
            {
                message += ", To Directory: " + name;
            }

            STAFTrace::trace(kSTAFTraceServiceRequest, message);
        }
    }

    // Add an entry to the FS Copy map

    STAFMutexSemLock fsCopyMapLock(fFSCopyMapSem);
    mapID = ++fMapID;
    fFSCopyMap[mapID] = FSCopyData(name, machine, direction, type,
                                   mode, fileSize);

    return kSTAFOk;
}

STAFRC_t STAFFSCopyManager::updateFileCopy1(
    unsigned int mapID, unsigned int fileSize)
{
    fFSCopyMap[mapID].fileSize = fileSize;
    fFSCopyMap[mapID].bytesCopied = 0;

    return kSTAFOk;
}

STAFRC_t STAFFSCopyManager::updateFileCopy(
    unsigned int mapID, unsigned int bytesCopied)
{
    fFSCopyMap[mapID].bytesCopied = bytesCopied;

    return kSTAFOk;
}


STAFRC_t STAFFSCopyManager::updateDirectoryCopy(
    unsigned int mapID, const STAFString &name, unsigned int mode,
    unsigned int fileSize)
{
    fFSCopyMap[mapID].entryName = name;
    fFSCopyMap[mapID].mode = mode;
    fFSCopyMap[mapID].fileSize = fileSize;
    fFSCopyMap[mapID].bytesCopied = 0;

    return kSTAFOk;
}


STAFRC_t STAFFSCopyManager::remove(unsigned int mapID)
{
    if (fFSCopyMap[mapID].direction == kSTAFFSCopyTo)
    {
        if (STAFTrace::doTrace(kSTAFTraceServiceResult) &&
            STAFServiceManager::doTraceService("FS"))
        {
            // Record a FS Copy Request Completed trace message

            STAFString message = "FS Copy Request Completed - From Machine: " +
                fFSCopyMap[mapID].machine;
        
            if (fFSCopyMap[mapID].type == kSTAFFSFileCopy)
            {
                message += ", To File: " + fFSCopyMap[mapID].name;

                if (fFSCopyMap[mapID].mode == kSTAFFSBinary)
                    message += ", Mode: Binary";
                else
                    message += ", Mode: Text";
            }
            else
            {
                message += ", To Directory: " + fFSCopyMap[mapID].name;
            }

            STAFTrace::trace(kSTAFTraceServiceResult, message);
        }
    }

    // Remove the entry from the FS Copy map

    STAFMutexSemLock fsCopyMapLock(fFSCopyMapSem);
    fFSCopyMap.erase(mapID);
    
    return kSTAFOk;
}


unsigned int STAFFSCopyManager::getMapID()
{
    return fMapID;
}


STAFFSCopyManager::FSCopyMap STAFFSCopyManager::getFSCopyMapCopy()
{
    STAFMutexSemLock fsCopyMapLock(fFSCopyMapSem);

    return fFSCopyMap;
}
